package be.cylab.java.wowa.training;

import com.owlike.genson.GenericType;
import com.owlike.genson.Genson;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.List;
import java.util.Random;

/**
 * Class Utils. Contain tools for Trainer algorithm
 */
public final class Utils {

    private Utils() {

    }

    /**
     * @param weights
     * @return
     */
    public static double[] normalizeWeights(final double[] weights) {
        double sum_weight = Utils.sumArrayElements(weights);
        double[] weights_normalized = new double[weights.length];
        for (int i = 0; i < weights.length; i++) {
            weights_normalized[i] = weights[i] / sum_weight;
        }
        return weights_normalized;
    }

    /**
     * @param solutions
     * @return
     */
    public static double findMaxDistance(
            final List<SolutionDistance> solutions) {
        double max = Double.NEGATIVE_INFINITY;
        for (SolutionDistance solution : solutions) {
            if (solution.getDistance() > max) {
                max = solution.getDistance();
            }
        }
        return max;
    }

    /**
     * @param solutions
     * @return
     */
    public static double findMinDistance(
            final List<SolutionDistance> solutions) {
        double min = Double.POSITIVE_INFINITY;
        for (SolutionDistance solution : solutions) {
            if (solution.getDistance() < min) {
                min = solution.getDistance();
            }
        }
        return min;
    }

    /**
     * @param solutions
     * @return
     */
    public static double sumTotalDistance(
            final List<SolutionDistance> solutions) {
        double sum = 0;
        for (SolutionDistance solution : solutions) {
            sum += solution.getDistance();
        }
        return sum;
    }

    /**
     * @param array
     * @return
     */
    public static double sumArrayElements(final double[] array) {
        float sum = 0;
        for (double weight : array) {
            sum += weight;
        }
        return sum;
    }

    /**
     * @param min
     * @param max
     * @return
     */
    public static int randomInteger(final int min, final int max) {
        if (min >= max) {
            throw new IllegalArgumentException("Max must be greater then min");
        }
        Random rnd = new Random();
        return rnd.nextInt((max - min) + 1) + min;
    }

    /**
     * Import a json file.
     * @param filename
     * @return
     * @throws IOException if we cannot read file
     */
    public static List<double[]> convertJsonToDataForTrainer(
            final String filename)
            throws IOException {

        Genson genson = new Genson();
        String data_json = new String(Files.readAllBytes(Paths.get(filename)));
        List<double[]> data = genson.deserialize(
                data_json,
                new GenericType<List<double[]>>() { });
        return data;
    }

    /**
     * Import result file.
     * @param filename
     * @return
     * @throws IOException if we cannot read file
     */
    public static double[] convertJsonToExpectedForTrainer(
            final String filename) throws IOException {

        Genson genson = new Genson();
        String expected_json = new String(
                Files.readAllBytes(Paths.get(filename)));
        double[] expected = genson.deserialize(expected_json, double[].class);
        return expected;
    }

}
